@@ -86,6 +86,7 @@ class BaseCore(object):
     """ The server core is the container for all Bcfg2 server logic
     and modules. All core implementations must inherit from
     ``BaseCore``. """
+    name = "core"
 
     def __init__(self, setup):  # pylint: disable=R0912,R0915
         """
@@ -188,6 +189,12 @@ class BaseCore(object):
         self.setup = setup
 
         atexit.register(self.shutdown)
+        #: if :func:`Bcfg2.Server.Core.shutdown` is called explicitly,
+        #: then :mod:`atexit` calls it *again*, so it gets called
+        #: twice.  This is potentially bad, so we use
+        #: :attr:`Bcfg2.Server.Core._running` as a flag to determine
+        #: if the core needs to be shutdown, and only do it once.
+        self._running = True
 
         #: Threading event to signal worker threads (e.g.,
         #: :attr:`fam_thread`) to shutdown
@@ -228,11 +235,11 @@ class BaseCore(object):
                                         verbosity=0)
                 self._database_available = True
             except ImproperlyConfigured:
-                err = sys.exc_info()[1]
-                self.logger.error("Django configuration problem: %s" % err)
+                self.logger.error("Django configuration problem: %s" %
+                                  sys.exc_info()[1])
             except:
-                err = sys.exc_info()[1]
-                self.logger.error("Database update failed: %s" % err)
+                self.logger.error("Database update failed: %s" %
+                                  sys.exc_info()[1])
 
             if do_chown and self._database_available:
                 try:
@@ -414,7 +421,7 @@ class BaseCore(object):
         :type plugin: string
         :returns: None
         """
-        self.logger.debug("Loading plugin %s" % plugin)
+        self.logger.debug("%s: Loading plugin %s" % (self.name, plugin))
         try:
             mod = getattr(__import__("Bcfg2.Server.Plugins.%s" %
                                      (plugin)).Server.Plugins, plugin)
@@ -450,14 +457,22 @@ class BaseCore(object):
 
     def shutdown(self):
         """ Perform plugin and FAM shutdown tasks. """
-        self.logger.info("Shutting down core...")
+        if not self._running:
+            self.logger.debug("%s: Core already shut down" % self.name)
+            return
+        self.logger.info("%s: Shutting down core..." % self.name)
         if not self.terminate.isSet():
             self.terminate.set()
-            self.fam.shutdown()
-            self.logger.info("FAM shut down")
-            for plugin in list(self.plugins.values()):
-                plugin.shutdown()
-            self.logger.info("All plugins shut down")
+        self._running = False
+        self.fam.shutdown()
+        self.logger.info("%s: FAM shut down" % self.name)
+        for plugin in list(self.plugins.values()):
+            plugin.shutdown()
+        self.logger.info("%s: All plugins shut down" % self.name)
+        if self._database_available:
+            from django import db
+            self.logger.info("%s: Closing database connection" % self.name)
+            db.close_connection()
 
     @property
     def metadata_cache_mode(self):
@@ -726,6 +741,11 @@ class BaseCore(object):
         self.validate_goals(meta, config)
 
         self.client_run_hook("end_client_run", meta)
+        if self._database_available:
+            from django import db
+            self.logger.debug("%s: Closing database connection" %
+                              threading.current_thread().name)
+            db.close_connection()
 
         sort_xml(config, key=lambda e: e.get('name'))
 
