@@ -16,7 +16,7 @@ from Bcfg2.Compat import CmpMixin, wraps
 from Bcfg2.Server.Plugin.base import Debuggable, Plugin
 from Bcfg2.Server.Plugin.interfaces import Generator
 from Bcfg2.Server.Plugin.exceptions import SpecificityError, \
-    PluginExecutionError
+    PluginExecutionError, PluginInitError
 
 try:
     import django  # pylint: disable=W0611
@@ -131,6 +131,19 @@ class DatabaseBacked(Plugin):
     #: conform to the possible values that function can handle.
     option = "use_database"
 
+    def __init__(self, core, datastore):
+        Plugin.__init__(self, core, datastore)
+        use_db = self.core.setup.cfp.getboolean(self.section,
+                                                self.option,
+                                                default=False)
+        if use_db and not HAS_DJANGO:
+            raise PluginInitError("%s.%s is True but Django not found" %
+                                  (self.section, self.option))
+        elif use_db and not self.core.database_available:
+            raise PluginInitError("%s.%s is True but the database is "
+                                  "unavailable due to prior errors" %
+                                  (self.section, self.option))
+
     def _section(self):
         """ The section to look in for :attr:`DatabaseBacked.option`
         """
@@ -146,10 +159,7 @@ class DatabaseBacked(Plugin):
                                                 default=False)
         if use_db and HAS_DJANGO and self.core.database_available:
             return True
-        elif not use_db:
-            return False
         else:
-            self.logger.error("%s is true but django not found" % self.option)
             return False
 
     @property
@@ -830,15 +840,10 @@ class XMLSrc(XMLFileBacked):
 
     def HandleEvent(self, _=None):
         """Read file upon update."""
-        try:
-            data = open(self.name).read()
-        except IOError:
-            msg = "Failed to read file %s: %s" % (self.name, sys.exc_info()[1])
-            self.logger.error(msg)
-            raise PluginExecutionError(msg)
         self.items = {}
         try:
-            xdata = lxml.etree.XML(data, parser=Bcfg2.Server.XMLParser)
+            xdata = lxml.etree.parse(self.name,
+                                     parser=Bcfg2.Server.XMLParser).getroot()
         except lxml.etree.XMLSyntaxError:
             msg = "Failed to parse file %s: %s" % (self.name,
                                                    sys.exc_info()[1])
@@ -855,8 +860,6 @@ class XMLSrc(XMLFileBacked):
                 self.logger.error(msg)
                 raise PluginExecutionError(msg)
 
-        del xdata, data
-
     def Cache(self, metadata):
         """Build a package dict for a given host."""
         if self.cache is None or self.cache[0] != metadata:
@@ -1142,7 +1145,8 @@ class SpecificData(object):
         except UnicodeDecodeError:
             self.data = open(self.name, mode='rb').read()
         except:  # pylint: disable=W0201
-            LOGGER.error("Failed to read file %s" % self.name)
+            LOGGER.error("Failed to read file %s: %s" % (self.name,
+                                                         sys.exc_info()[1]))
 
 
 class EntrySet(Debuggable):
